#!/bin/bash

set -xuve

# Constants
QAT_URL="eqc-cms.mcovh.duckdns.org"
USER="admin"
PASSWD="1234"
UNIT_FIELD_NAME="field_tb_cdt_unit_test_results"
INT_FIELD_NAME="field_tb_tcd_in_log_file_output"
CDM_FIELD_NAME="field_tb_tcd_out_log_file_output"

# Parameters
while getopts "f:t:U:I:o:" o; do
    case "${o}" in
        f)
            FAMILY=${OPTARG}
            ;;
        t)
            TOOL=${OPTARG}
            ;;
        U)
            UNIT_TEST=${OPTARG}
            ;;
        I)
            INT_TEST_CONFIG=${OPTARG}
            ;;
        o)
            PDF_OUTPUT=${OPTARG}
            ;;
        *)
	    echo "Exiting"
	    exit
            ;;
    esac
done

# Processing procedures
function do_unit_test() {
    UNIT_LOG=$TMP_DIR/unit.json
    UNIT_TXT_1=$TMP_DIR/U1.txt
    UNIT_TXT_2=$TMP_DIR/U2.txt
    UNIT_METADATA=$TMP_DIR/unit.metadata
    UNIT_PDF=$TMP_DIR/$FAMILY-$TOOL-unit.pdf
    echo "Running unit test. Logging to: " $UNIT_LOG
    pytest --report-log=$UNIT_LOG $UNIT_TEST
    echo "Parsing unit testing output log"
    python3 $C3S512_WP1_DATACHECKER/dqc_wrapper/scripts/json2txt_unit.py $UNIT_LOG  $UNIT_TXT_1 $UNIT_TXT_2
    echo "Generating PDF: $UNIT_PDF"
    python3 $C3S512_WP1_DATACHECKER/dqc_wrapper/scripts/txt2pdf_customTitle.py $UNIT_TXT_1 $UNIT_TXT_2 $UNIT_PDF "Summary of unit tests"
    echo -e \
"    {
      \"field_name\": \"$UNIT_FIELD_NAME\",
      \"type\": \"file\",
      \"value\": \"unit\"
    }"> $UNIT_METADATA
}

function do_integration_and_compliance_test() {
    INT_FINAL_CONFIG=$TMP_DIR/int.yaml
    INT_TXT_1=$TMP_DIR/I1.txt
    INT_TXT_2=$TMP_DIR/I2.txt
    INT_METADATA=$TMP_DIR/int.metadata
    INT_PDF=$TMP_DIR/$FAMILY-$TOOL-integration.pdf
    echo "Setting integration tests logging output"
    echo "logs_location:" $TMP_DIR > $TMP_DIR/config_header
    tail -n +3 $INT_TEST_CONFIG > $TMP_DIR/config_body
    cat $TMP_DIR/config_header $TMP_DIR/config_body > $INT_FINAL_CONFIG
    echo "Final configuration file: $INT_FINAL_CONFIG"
    echo "Running integration test"
    run_toolbox_integration_tests -c $INT_FINAL_CONFIG -t $TOOL
    echo "Parsing integration testing output log"
    python3 $C3S512_WP1_DATACHECKER/dqc_wrapper/scripts/json2txt_integration.py $TMP_DIR/*_integration_test.json  $INT_TXT_1 $INT_TXT_2
    echo "Generating PDF: $INT_PDF"
    python3 $C3S512_WP1_DATACHECKER/dqc_wrapper/scripts/txt2pdf_customTitle.py $INT_TXT_1 $INT_TXT_2 $INT_PDF "Summary of integration tests"
    echo -e \
"    {\n
      \"field_name\": \"$INT_FIELD_NAME\",
      \"type\": \"file\",
      \"value\": \"integration\"
    }"> $INT_METADATA
    
    ## Do CDM test
    CDM_TXT_1=$TMP_DIR/C1.txt
    CDM_TXT_2=$TMP_DIR/C2.txt
    CDM_METADATA=$TMP_DIR/cdm.metadata
    CDM_PDF=$TMP_DIR/$FAMILY-$TOOL-compliance.pdf
    echo "Parsing CDM check output log"
    python3 $C3S512_WP1_DATACHECKER/dqc_wrapper/scripts/json2txt_cdm.py $TMP_DIR/*_integration_test.json  $CDM_TXT_1 $CDM_TXT_2
    echo "Generating PDF: $CDM_PDF"
    python3 $C3S512_WP1_DATACHECKER/dqc_wrapper/scripts/txt2pdf_customTitle.py $CDM_TXT_1 $CDM_TXT_2 $CDM_PDF "Summary of CDM compliance checks"
    echo -e \
"    {\n
      \"field_name\": \"$CDM_FIELD_NAME\",
      \"type\": \"file\",
      \"value\": \"compliance\"
    }"> $CDM_METADATA
}

if [[ -z ${CONDA_DEFAULT_ENV:+x} || $CONDA_DEFAULT_ENV != "toolbox_env" ]]; then
    echo "The conda environment \"toolbox_env\" is required and not activated. Please, execute \"conda activate toolbox_env\" before running this program. I wouldn't mind doing it myself..."
    echo "but I can't (https://github.com/conda/conda/issues/7980#issuecomment-492788779)."
    echo "Exiting."
elif [ -z ${FAMILY+x} ]; then
    echo "Mandatory parameter FAMILY (-f) not set. Exiting."
    exit
elif [ -z ${TOOL+x} ]; then
    echo "Mandatory parameter TOOL (-t) not set. Exiting"
    exit
elif [ -z ${UNIT_TEST+x} ]; then
    if [ -z ${INT_TEST_CONFIG+x} ]; then
	echo "Unit testing placeholder. No testing performed"
	if [ -z ${PDF_OUTPUT+x} ]; then
	    echo "Output: placeholder PDF"
	    cp /data/docs/not-available-unit.pdf $PDF_OUTPUT
	fi
    else
	echo "Cannot perform integration test without unit test. Exiting"
	exit
    fi
else
    TMP_DIR=`mktemp -d`
    C3S512_WP1_DATACHECKER="/data/dtrujill/git/c3s512-wp1-datachecker"

    METADATA_FILE=$TMP_DIR/metadata.json
    METADATA_START_FILE=$TMP_DIR/metadata.start.json
    METADATA_END_FILE=$TMP_DIR/metadata.end.json
    echo -e \
"{
  \"qar_name\": \"cdstoolbox-$FAMILY-$TOOL\",
  \"changes\": [" > $METADATA_START_FILE

    echo -e  \
"  ]
}" > $METADATA_END_FILE
    
    ## Prepare environment
    #conda activate toolbox_env
    
    if [ -z ${INT_TEST_CONFIG+x} ]; then
	echo "Running unit testing only"
	do_unit_test

	cat $METADATA_START_FILE $UNIT_METADATA $METADATA_END_FILE > $METADATA_FILE

	curl -u $USER:$PASSWD -s -F "metadata=@$METADATA_FILE" -F "unit=@$UNIT_PDF" $QAT_URL/fillQAR > $TMP_DIR/qar_filler_result.html
	
        grep "Result: " $TMP_DIR/qar_filler_result.html || grep "permiso" $TMP_DIR/qar_filler_result.html
    else
	echo "Running all tests"
	do_unit_test
	do_integration_and_compliance_test

	COMMA_FILE=$TMP_DIR/comma.txt
	echo "    ," > $COMMA_FILE

	cat $METADATA_START_FILE $UNIT_METADATA $COMMA_FILE $INT_METADATA $COMMA_FILE $CDM_METADATA $METADATA_END_FILE > $METADATA_FILE

	curl -u $USER:$PASSWD -F "metadata=@$METADATA_FILE" -F "unit=@$UNIT_PDF" -F "integration=@$INT_PDF" -F "compliance=@$CDM_PDF" $QAT_URL/fillQAR > $TMP_DIR/qar_filler_result.html
	
	grep "Result: " $TMP_DIR/qar_filler_result.html || grep "permiso" $TMP_DIR/qar_filler_result.html 
    fi


    if [ ! -z ${PDF_OUTPUT:+x} ]; then
	## Copy PDFs to selected directory
	cp $TMP_DIR/*.pdf $PDF_OUTPUT
    fi
fi
